# Copyright © 2012-2023 jrnl contributors
# License: https://www.gnu.org/licenses/gpl-3.0.html

try:
    from jrnl.__version__ import __version__
except ImportError:
    __version__ = "source"
__title__ = "jrnl"
